/*
 * Copyright 2019 Google LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.google.android.overtheair.spi.v1;

import com.google.android.overtheair.spi.v1.stub.OverTheAirServiceStub;
import com.google.android.overtheair.spi.v1.stub.OverTheAirServiceStubSettings;
import com.google.android.overtheair.v1.Chart;
import com.google.android.overtheair.v1.ChartOptions;
import com.google.android.overtheair.v1.Config;
import com.google.android.overtheair.v1.CreateConfigRequest;
import com.google.android.overtheair.v1.CreateGroupMemberRequest;
import com.google.android.overtheair.v1.CreateGroupMembersRequest;
import com.google.android.overtheair.v1.CreateGroupMembersResponse;
import com.google.android.overtheair.v1.CreateGroupRequest;
import com.google.android.overtheair.v1.DeleteConfigRequest;
import com.google.android.overtheair.v1.DeleteConfigResponse;
import com.google.android.overtheair.v1.DeleteGroupMemberRequest;
import com.google.android.overtheair.v1.DeleteGroupMemberResponse;
import com.google.android.overtheair.v1.DeleteGroupMembersRequest;
import com.google.android.overtheair.v1.DeleteGroupMembersResponse;
import com.google.android.overtheair.v1.DeleteGroupRequest;
import com.google.android.overtheair.v1.DeleteGroupResponse;
import com.google.android.overtheair.v1.DeletePackageRequest;
import com.google.android.overtheair.v1.DeletePackageResponse;
import com.google.android.overtheair.v1.Deployment;
import com.google.android.overtheair.v1.DisableAllEnabledGroupsOfConfigRequest;
import com.google.android.overtheair.v1.DisableAllEnabledGroupsOfConfigResponse;
import com.google.android.overtheair.v1.DisableGroupsRequest;
import com.google.android.overtheair.v1.DisableGroupsResponse;
import com.google.android.overtheair.v1.EnableGroupsRequest;
import com.google.android.overtheair.v1.EnableGroupsResponse;
import com.google.android.overtheair.v1.GetConfigRequest;
import com.google.android.overtheair.v1.GetDeploymentRequest;
import com.google.android.overtheair.v1.GetGroupChartRequest;
import com.google.android.overtheair.v1.GetGroupMemberRequest;
import com.google.android.overtheair.v1.GetGroupRequest;
import com.google.android.overtheair.v1.GetPackageRequest;
import com.google.android.overtheair.v1.Group;
import com.google.android.overtheair.v1.GroupMember;
import com.google.android.overtheair.v1.ListConfigPackagesRequest;
import com.google.android.overtheair.v1.ListConfigPackagesResponse;
import com.google.android.overtheair.v1.ListConfigsRequest;
import com.google.android.overtheair.v1.ListConfigsResponse;
import com.google.android.overtheair.v1.ListConfigsUsingPackageRequest;
import com.google.android.overtheair.v1.ListConfigsUsingPackageResponse;
import com.google.android.overtheair.v1.ListDeploymentsRequest;
import com.google.android.overtheair.v1.ListDeploymentsResponse;
import com.google.android.overtheair.v1.ListGroupChartsRequest;
import com.google.android.overtheair.v1.ListGroupChartsResponse;
import com.google.android.overtheair.v1.ListGroupConfigsRequest;
import com.google.android.overtheair.v1.ListGroupConfigsResponse;
import com.google.android.overtheair.v1.ListGroupMembersRequest;
import com.google.android.overtheair.v1.ListGroupMembersResponse;
import com.google.android.overtheair.v1.ListGroupsRequest;
import com.google.android.overtheair.v1.ListGroupsResponse;
import com.google.android.overtheair.v1.ListPackagesRequest;
import com.google.android.overtheair.v1.ListPackagesResponse;
import com.google.android.overtheair.v1.PackageConfig;
import com.google.android.overtheair.v1.UpdateConfigPackagesRequest;
import com.google.android.overtheair.v1.UpdateConfigPackagesResponse;
import com.google.android.overtheair.v1.UpdateConfigRequest;
import com.google.android.overtheair.v1.UpdateDeploymentRequest;
import com.google.android.overtheair.v1.UpdateGroupConfigsRequest;
import com.google.android.overtheair.v1.UpdateGroupConfigsResponse;
import com.google.android.overtheair.v1.UpdateGroupRequest;
import com.google.android.overtheair.v1.UpdatePackageRequest;
import com.google.android.overtheair.v1.VersionTag;
import com.google.android.overtheair.v1.View;
import com.google.api.core.BetaApi;
import com.google.api.gax.core.BackgroundResource;
import com.google.api.gax.rpc.UnaryCallable;
import com.google.api.pathtemplate.PathTemplate;
import java.io.IOException;
import java.util.List;
import java.util.concurrent.TimeUnit;
import javax.annotation.Generated;

// AUTO-GENERATED DOCUMENTATION AND SERVICE
/**
 * Service Description: ## Service for Android Over the Air(OTA).
 *
 * <p>## ---- deployment related rpcs ---
 *
 * <p>This class provides the ability to make remote calls to the backing service through method
 * calls that map to API methods. Sample code to get started:
 *
 * <pre>
 * <code>
 * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
 *   String formattedName = OverTheAirServiceClient.formatDeploymentName("[DEPLOYMENT]");
 *   View view = View.BASIC;
 *   Deployment response = overTheAirServiceClient.getDeployment(formattedName, view);
 * }
 * </code>
 * </pre>
 *
 * <p>Note: close() needs to be called on the overTheAirServiceClient object to clean up resources
 * such as threads. In the example above, try-with-resources is used, which automatically calls
 * close().
 *
 * <p>The surface of this class includes several types of Java methods for each of the API's
 * methods:
 *
 * <ol>
 *   <li> A "flattened" method. With this type of method, the fields of the request type have been
 *       converted into function parameters. It may be the case that not all fields are available as
 *       parameters, and not every API method will have a flattened method entry point.
 *   <li> A "request object" method. This type of method only takes one parameter, a request object,
 *       which must be constructed before the call. Not every API method will have a request object
 *       method.
 *   <li> A "callable" method. This type of method takes no parameters and returns an immutable API
 *       callable object, which can be used to initiate calls to the service.
 * </ol>
 *
 * <p>See the individual methods for example code.
 *
 * <p>Many parameters require resource names to be formatted in a particular way. To assist with
 * these names, this class includes a format method for each type of name, and additionally a parse
 * method to extract the individual identifiers contained within names that are returned.
 *
 * <p>This class can be customized by passing in a custom instance of OverTheAirServiceSettings to
 * create(). For example:
 *
 * <p>To customize credentials:
 *
 * <pre>
 * <code>
 * OverTheAirServiceSettings overTheAirServiceSettings =
 *     OverTheAirServiceSettings.newBuilder()
 *         .setCredentialsProvider(FixedCredentialsProvider.create(myCredentials))
 *         .build();
 * OverTheAirServiceClient overTheAirServiceClient =
 *     OverTheAirServiceClient.create(overTheAirServiceSettings);
 * </code>
 * </pre>
 *
 * To customize the endpoint:
 *
 * <pre>
 * <code>
 * OverTheAirServiceSettings overTheAirServiceSettings =
 *     OverTheAirServiceSettings.newBuilder().setEndpoint(myEndpoint).build();
 * OverTheAirServiceClient overTheAirServiceClient =
 *     OverTheAirServiceClient.create(overTheAirServiceSettings);
 * </code>
 * </pre>
 */
@Generated("by gapic-generator")
@BetaApi
public class OverTheAirServiceClient implements BackgroundResource {
  private final OverTheAirServiceSettings settings;
  private final OverTheAirServiceStub stub;

  private static final PathTemplate DEPLOYMENT_PATH_TEMPLATE =
      PathTemplate.createWithoutUrlEncoding("deployments/{deployment}");

  private static final PathTemplate CONFIG_PATH_TEMPLATE =
      PathTemplate.createWithoutUrlEncoding("deployments/{deployment}/configs/{config}");

  private static final PathTemplate GROUP_PATH_TEMPLATE =
      PathTemplate.createWithoutUrlEncoding("deployments/{deployment}/groups/{group}");

  private static final PathTemplate CHART_PATH_TEMPLATE =
      PathTemplate.createWithoutUrlEncoding(
          "deployments/{deployment}/groups/{group}/charts/{chart}");

  private static final PathTemplate MEMBER_PATH_TEMPLATE =
      PathTemplate.createWithoutUrlEncoding(
          "deployments/{deployment}/groups/{group}/members/{member}");

  private static final PathTemplate PACKAGE_PATH_PATH_TEMPLATE =
      PathTemplate.createWithoutUrlEncoding("deployments/{deployment}/packages/{package_path=**}");

  /**
   * Formats a string containing the fully-qualified path to represent a deployment resource.
   *
   * @deprecated Use the {@link DeploymentName} class instead.
   */
  @Deprecated
  public static final String formatDeploymentName(String deployment) {
    return DEPLOYMENT_PATH_TEMPLATE.instantiate("deployment", deployment);
  }

  /**
   * Formats a string containing the fully-qualified path to represent a config resource.
   *
   * @deprecated Use the {@link ConfigName} class instead.
   */
  @Deprecated
  public static final String formatConfigName(String deployment, String config) {
    return CONFIG_PATH_TEMPLATE.instantiate(
        "deployment", deployment,
        "config", config);
  }

  /**
   * Formats a string containing the fully-qualified path to represent a group resource.
   *
   * @deprecated Use the {@link GroupName} class instead.
   */
  @Deprecated
  public static final String formatGroupName(String deployment, String group) {
    return GROUP_PATH_TEMPLATE.instantiate(
        "deployment", deployment,
        "group", group);
  }

  /**
   * Formats a string containing the fully-qualified path to represent a chart resource.
   *
   * @deprecated Use the {@link ChartName} class instead.
   */
  @Deprecated
  public static final String formatChartName(String deployment, String group, String chart) {
    return CHART_PATH_TEMPLATE.instantiate(
        "deployment", deployment,
        "group", group,
        "chart", chart);
  }

  /**
   * Formats a string containing the fully-qualified path to represent a member resource.
   *
   * @deprecated Use the {@link MemberName} class instead.
   */
  @Deprecated
  public static final String formatMemberName(String deployment, String group, String member) {
    return MEMBER_PATH_TEMPLATE.instantiate(
        "deployment", deployment,
        "group", group,
        "member", member);
  }

  /**
   * Formats a string containing the fully-qualified path to represent a package_path resource.
   *
   * @deprecated Use the {@link PackagePathName} class instead.
   */
  @Deprecated
  public static final String formatPackagePathName(String deployment, String packagePath) {
    return PACKAGE_PATH_PATH_TEMPLATE.instantiate(
        "deployment", deployment,
        "package_path", packagePath);
  }

  /**
   * Parses the deployment from the given fully-qualified path which represents a deployment
   * resource.
   *
   * @deprecated Use the {@link DeploymentName} class instead.
   */
  @Deprecated
  public static final String parseDeploymentFromDeploymentName(String deploymentName) {
    return DEPLOYMENT_PATH_TEMPLATE.parse(deploymentName).get("deployment");
  }

  /**
   * Parses the deployment from the given fully-qualified path which represents a config resource.
   *
   * @deprecated Use the {@link ConfigName} class instead.
   */
  @Deprecated
  public static final String parseDeploymentFromConfigName(String configName) {
    return CONFIG_PATH_TEMPLATE.parse(configName).get("deployment");
  }

  /**
   * Parses the config from the given fully-qualified path which represents a config resource.
   *
   * @deprecated Use the {@link ConfigName} class instead.
   */
  @Deprecated
  public static final String parseConfigFromConfigName(String configName) {
    return CONFIG_PATH_TEMPLATE.parse(configName).get("config");
  }

  /**
   * Parses the deployment from the given fully-qualified path which represents a group resource.
   *
   * @deprecated Use the {@link GroupName} class instead.
   */
  @Deprecated
  public static final String parseDeploymentFromGroupName(String groupName) {
    return GROUP_PATH_TEMPLATE.parse(groupName).get("deployment");
  }

  /**
   * Parses the group from the given fully-qualified path which represents a group resource.
   *
   * @deprecated Use the {@link GroupName} class instead.
   */
  @Deprecated
  public static final String parseGroupFromGroupName(String groupName) {
    return GROUP_PATH_TEMPLATE.parse(groupName).get("group");
  }

  /**
   * Parses the deployment from the given fully-qualified path which represents a chart resource.
   *
   * @deprecated Use the {@link ChartName} class instead.
   */
  @Deprecated
  public static final String parseDeploymentFromChartName(String chartName) {
    return CHART_PATH_TEMPLATE.parse(chartName).get("deployment");
  }

  /**
   * Parses the group from the given fully-qualified path which represents a chart resource.
   *
   * @deprecated Use the {@link ChartName} class instead.
   */
  @Deprecated
  public static final String parseGroupFromChartName(String chartName) {
    return CHART_PATH_TEMPLATE.parse(chartName).get("group");
  }

  /**
   * Parses the chart from the given fully-qualified path which represents a chart resource.
   *
   * @deprecated Use the {@link ChartName} class instead.
   */
  @Deprecated
  public static final String parseChartFromChartName(String chartName) {
    return CHART_PATH_TEMPLATE.parse(chartName).get("chart");
  }

  /**
   * Parses the deployment from the given fully-qualified path which represents a member resource.
   *
   * @deprecated Use the {@link MemberName} class instead.
   */
  @Deprecated
  public static final String parseDeploymentFromMemberName(String memberName) {
    return MEMBER_PATH_TEMPLATE.parse(memberName).get("deployment");
  }

  /**
   * Parses the group from the given fully-qualified path which represents a member resource.
   *
   * @deprecated Use the {@link MemberName} class instead.
   */
  @Deprecated
  public static final String parseGroupFromMemberName(String memberName) {
    return MEMBER_PATH_TEMPLATE.parse(memberName).get("group");
  }

  /**
   * Parses the member from the given fully-qualified path which represents a member resource.
   *
   * @deprecated Use the {@link MemberName} class instead.
   */
  @Deprecated
  public static final String parseMemberFromMemberName(String memberName) {
    return MEMBER_PATH_TEMPLATE.parse(memberName).get("member");
  }

  /**
   * Parses the deployment from the given fully-qualified path which represents a package_path
   * resource.
   *
   * @deprecated Use the {@link PackagePathName} class instead.
   */
  @Deprecated
  public static final String parseDeploymentFromPackagePathName(String packagePathName) {
    return PACKAGE_PATH_PATH_TEMPLATE.parse(packagePathName).get("deployment");
  }

  /**
   * Parses the package_path from the given fully-qualified path which represents a package_path
   * resource.
   *
   * @deprecated Use the {@link PackagePathName} class instead.
   */
  @Deprecated
  public static final String parsePackagePathFromPackagePathName(String packagePathName) {
    return PACKAGE_PATH_PATH_TEMPLATE.parse(packagePathName).get("package_path");
  }

  /** Constructs an instance of OverTheAirServiceClient with default settings. */
  public static final OverTheAirServiceClient create() throws IOException {
    return create(OverTheAirServiceSettings.newBuilder().build());
  }

  /**
   * Constructs an instance of OverTheAirServiceClient, using the given settings. The channels are
   * created based on the settings passed in, or defaults for any settings that are not set.
   */
  public static final OverTheAirServiceClient create(OverTheAirServiceSettings settings)
      throws IOException {
    return new OverTheAirServiceClient(settings);
  }

  /**
   * Constructs an instance of OverTheAirServiceClient, using the given stub for making calls. This
   * is for advanced usage - prefer to use OverTheAirServiceSettings}.
   */
  @BetaApi("A restructuring of stub classes is planned, so this may break in the future")
  public static final OverTheAirServiceClient create(OverTheAirServiceStub stub) {
    return new OverTheAirServiceClient(stub);
  }

  /**
   * Constructs an instance of OverTheAirServiceClient, using the given settings. This is protected
   * so that it is easy to make a subclass, but otherwise, the static factory methods should be
   * preferred.
   */
  protected OverTheAirServiceClient(OverTheAirServiceSettings settings) throws IOException {
    this.settings = settings;
    this.stub = ((OverTheAirServiceStubSettings) settings.getStubSettings()).createStub();
  }

  @BetaApi("A restructuring of stub classes is planned, so this may break in the future")
  protected OverTheAirServiceClient(OverTheAirServiceStub stub) {
    this.settings = null;
    this.stub = stub;
  }

  public final OverTheAirServiceSettings getSettings() {
    return settings;
  }

  @BetaApi("A restructuring of stub classes is planned, so this may break in the future")
  public OverTheAirServiceStub getStub() {
    return stub;
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Gets a [Deployment][google.android.overtheair.v1.Deployment]. Returns `NOT_FOUND` if the
   * deployment does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatDeploymentName("[DEPLOYMENT]");
   *   View view = View.BASIC;
   *   Deployment response = overTheAirServiceClient.getDeployment(formattedName, view);
   * }
   * </code></pre>
   *
   * @param name Relative resource name. For example: "deployments/deployment_x"
   * @param view Optional. Specify whether basic or detailed information of the deployment should be
   *     returned.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final Deployment getDeployment(String name, View view) {
    DEPLOYMENT_PATH_TEMPLATE.validate(name, "getDeployment");
    GetDeploymentRequest request =
        GetDeploymentRequest.newBuilder().setName(name).setView(view).build();
    return getDeployment(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Gets a [Deployment][google.android.overtheair.v1.Deployment]. Returns `NOT_FOUND` if the
   * deployment does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatDeploymentName("[DEPLOYMENT]");
   *   GetDeploymentRequest request = GetDeploymentRequest.newBuilder()
   *     .setName(formattedName)
   *     .build();
   *   Deployment response = overTheAirServiceClient.getDeployment(request);
   * }
   * </code></pre>
   *
   * @param request The request object containing all of the parameters for the API call.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final Deployment getDeployment(GetDeploymentRequest request) {
    return getDeploymentCallable().call(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Gets a [Deployment][google.android.overtheair.v1.Deployment]. Returns `NOT_FOUND` if the
   * deployment does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatDeploymentName("[DEPLOYMENT]");
   *   GetDeploymentRequest request = GetDeploymentRequest.newBuilder()
   *     .setName(formattedName)
   *     .build();
   *   ApiFuture&lt;Deployment&gt; future = overTheAirServiceClient.getDeploymentCallable().futureCall(request);
   *   // Do something
   *   Deployment response = future.get();
   * }
   * </code></pre>
   */
  public final UnaryCallable<GetDeploymentRequest, Deployment> getDeploymentCallable() {
    return stub.getDeploymentCallable();
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Lists [deployments][google.android.overtheair.v1.Deployment].
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   View view = View.BASIC;
   *   ListDeploymentsResponse response = overTheAirServiceClient.listDeployments(view);
   * }
   * </code></pre>
   *
   * @param view Optional. Specify whether basic or detailed information of each deployment should
   *     be returned.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final ListDeploymentsResponse listDeployments(View view) {

    ListDeploymentsRequest request = ListDeploymentsRequest.newBuilder().setView(view).build();
    return listDeployments(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Lists [deployments][google.android.overtheair.v1.Deployment].
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   ListDeploymentsRequest request = ListDeploymentsRequest.newBuilder().build();
   *   ListDeploymentsResponse response = overTheAirServiceClient.listDeployments(request);
   * }
   * </code></pre>
   *
   * @param request The request object containing all of the parameters for the API call.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final ListDeploymentsResponse listDeployments(ListDeploymentsRequest request) {
    return listDeploymentsCallable().call(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Lists [deployments][google.android.overtheair.v1.Deployment].
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   ListDeploymentsRequest request = ListDeploymentsRequest.newBuilder().build();
   *   ApiFuture&lt;ListDeploymentsResponse&gt; future = overTheAirServiceClient.listDeploymentsCallable().futureCall(request);
   *   // Do something
   *   ListDeploymentsResponse response = future.get();
   * }
   * </code></pre>
   */
  public final UnaryCallable<ListDeploymentsRequest, ListDeploymentsResponse>
      listDeploymentsCallable() {
    return stub.listDeploymentsCallable();
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Updates a [deployment][google.android.overtheair.v1.Deployment].
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatDeploymentName("[DEPLOYMENT]");
   *   Deployment deployment = Deployment.newBuilder().build();
   *   VersionTag versionTag = VersionTag.newBuilder().build();
   *   Deployment response = overTheAirServiceClient.updateDeployment(formattedName, deployment, versionTag);
   * }
   * </code></pre>
   *
   * @param name Relative resource name. For example: "deployments/deployment_x"
   * @param deployment Required. Only the deployment description can be updated.
   * @param versionTag Required. The deployment to be updated should match this version. If another
   *     client updates the existing instance concurrently, this update operation will fail.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final Deployment updateDeployment(
      String name, Deployment deployment, VersionTag versionTag) {
    DEPLOYMENT_PATH_TEMPLATE.validate(name, "updateDeployment");
    UpdateDeploymentRequest request =
        UpdateDeploymentRequest.newBuilder()
            .setName(name)
            .setDeployment(deployment)
            .setVersionTag(versionTag)
            .build();
    return updateDeployment(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Updates a [deployment][google.android.overtheair.v1.Deployment].
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatDeploymentName("[DEPLOYMENT]");
   *   Deployment deployment = Deployment.newBuilder().build();
   *   VersionTag versionTag = VersionTag.newBuilder().build();
   *   UpdateDeploymentRequest request = UpdateDeploymentRequest.newBuilder()
   *     .setName(formattedName)
   *     .setDeployment(deployment)
   *     .setVersionTag(versionTag)
   *     .build();
   *   Deployment response = overTheAirServiceClient.updateDeployment(request);
   * }
   * </code></pre>
   *
   * @param request The request object containing all of the parameters for the API call.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final Deployment updateDeployment(UpdateDeploymentRequest request) {
    return updateDeploymentCallable().call(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Updates a [deployment][google.android.overtheair.v1.Deployment].
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatDeploymentName("[DEPLOYMENT]");
   *   Deployment deployment = Deployment.newBuilder().build();
   *   VersionTag versionTag = VersionTag.newBuilder().build();
   *   UpdateDeploymentRequest request = UpdateDeploymentRequest.newBuilder()
   *     .setName(formattedName)
   *     .setDeployment(deployment)
   *     .setVersionTag(versionTag)
   *     .build();
   *   ApiFuture&lt;Deployment&gt; future = overTheAirServiceClient.updateDeploymentCallable().futureCall(request);
   *   // Do something
   *   Deployment response = future.get();
   * }
   * </code></pre>
   */
  public final UnaryCallable<UpdateDeploymentRequest, Deployment> updateDeploymentCallable() {
    return stub.updateDeploymentCallable();
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Gets a [package][google.android.overtheair.v1.Package]. Returns `NOT_FOUND` if the matching
   * path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatPackagePathName("[DEPLOYMENT]", "[PACKAGE_PATH]");
   *   View view = View.BASIC;
   *   com.google.android.overtheair.v1.Package response = overTheAirServiceClient.getPackage(formattedName, view);
   * }
   * </code></pre>
   *
   * @param name Relative resource name. For example: "deployments/deployment_x/packages/package_a"
   * @param view Optional. Specify whether basic or detailed information of the package should be
   *     returned.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final com.google.android.overtheair.v1.Package getPackage(String name, View view) {
    PACKAGE_PATH_PATH_TEMPLATE.validate(name, "getPackage");
    GetPackageRequest request = GetPackageRequest.newBuilder().setName(name).setView(view).build();
    return getPackage(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Gets a [package][google.android.overtheair.v1.Package]. Returns `NOT_FOUND` if the matching
   * path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatPackagePathName("[DEPLOYMENT]", "[PACKAGE_PATH]");
   *   GetPackageRequest request = GetPackageRequest.newBuilder()
   *     .setName(formattedName)
   *     .build();
   *   com.google.android.overtheair.v1.Package response = overTheAirServiceClient.getPackage(request);
   * }
   * </code></pre>
   *
   * @param request The request object containing all of the parameters for the API call.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final com.google.android.overtheair.v1.Package getPackage(GetPackageRequest request) {
    return getPackageCallable().call(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Gets a [package][google.android.overtheair.v1.Package]. Returns `NOT_FOUND` if the matching
   * path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatPackagePathName("[DEPLOYMENT]", "[PACKAGE_PATH]");
   *   GetPackageRequest request = GetPackageRequest.newBuilder()
   *     .setName(formattedName)
   *     .build();
   *   ApiFuture&lt;com.google.android.overtheair.v1.Package&gt; future = overTheAirServiceClient.getPackageCallable().futureCall(request);
   *   // Do something
   *   com.google.android.overtheair.v1.Package response = future.get();
   * }
   * </code></pre>
   */
  public final UnaryCallable<GetPackageRequest, com.google.android.overtheair.v1.Package>
      getPackageCallable() {
    return stub.getPackageCallable();
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Lists [packages][google.android.overtheair.v1.Package].
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatDeploymentName("[DEPLOYMENT]");
   *   View view = View.BASIC;
   *   ListPackagesResponse response = overTheAirServiceClient.listPackages(formattedName, view);
   * }
   * </code></pre>
   *
   * @param name Relative resource name. For example: "deployments/deployment_x"
   * @param view Optional. Specify whether basic or detailed information of each package should be
   *     returned.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final ListPackagesResponse listPackages(String name, View view) {
    DEPLOYMENT_PATH_TEMPLATE.validate(name, "listPackages");
    ListPackagesRequest request =
        ListPackagesRequest.newBuilder().setName(name).setView(view).build();
    return listPackages(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Lists [packages][google.android.overtheair.v1.Package].
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatDeploymentName("[DEPLOYMENT]");
   *   ListPackagesRequest request = ListPackagesRequest.newBuilder()
   *     .setName(formattedName)
   *     .build();
   *   ListPackagesResponse response = overTheAirServiceClient.listPackages(request);
   * }
   * </code></pre>
   *
   * @param request The request object containing all of the parameters for the API call.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final ListPackagesResponse listPackages(ListPackagesRequest request) {
    return listPackagesCallable().call(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Lists [packages][google.android.overtheair.v1.Package].
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatDeploymentName("[DEPLOYMENT]");
   *   ListPackagesRequest request = ListPackagesRequest.newBuilder()
   *     .setName(formattedName)
   *     .build();
   *   ApiFuture&lt;ListPackagesResponse&gt; future = overTheAirServiceClient.listPackagesCallable().futureCall(request);
   *   // Do something
   *   ListPackagesResponse response = future.get();
   * }
   * </code></pre>
   */
  public final UnaryCallable<ListPackagesRequest, ListPackagesResponse> listPackagesCallable() {
    return stub.listPackagesCallable();
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Deletes a [package][google.android.overtheair.v1.Package]. Returns `NOT_FOUND` if the matching
   * path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatPackagePathName("[DEPLOYMENT]", "[PACKAGE_PATH]");
   *   DeletePackageResponse response = overTheAirServiceClient.deletePackage(formattedName);
   * }
   * </code></pre>
   *
   * @param name Relative resource name. For example: "deployments/deployment_x/packages/package_a"
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final DeletePackageResponse deletePackage(String name) {
    PACKAGE_PATH_PATH_TEMPLATE.validate(name, "deletePackage");
    DeletePackageRequest request = DeletePackageRequest.newBuilder().setName(name).build();
    return deletePackage(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Deletes a [package][google.android.overtheair.v1.Package]. Returns `NOT_FOUND` if the matching
   * path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatPackagePathName("[DEPLOYMENT]", "[PACKAGE_PATH]");
   *   DeletePackageRequest request = DeletePackageRequest.newBuilder()
   *     .setName(formattedName)
   *     .build();
   *   DeletePackageResponse response = overTheAirServiceClient.deletePackage(request);
   * }
   * </code></pre>
   *
   * @param request The request object containing all of the parameters for the API call.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final DeletePackageResponse deletePackage(DeletePackageRequest request) {
    return deletePackageCallable().call(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Deletes a [package][google.android.overtheair.v1.Package]. Returns `NOT_FOUND` if the matching
   * path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatPackagePathName("[DEPLOYMENT]", "[PACKAGE_PATH]");
   *   DeletePackageRequest request = DeletePackageRequest.newBuilder()
   *     .setName(formattedName)
   *     .build();
   *   ApiFuture&lt;DeletePackageResponse&gt; future = overTheAirServiceClient.deletePackageCallable().futureCall(request);
   *   // Do something
   *   DeletePackageResponse response = future.get();
   * }
   * </code></pre>
   */
  public final UnaryCallable<DeletePackageRequest, DeletePackageResponse> deletePackageCallable() {
    return stub.deletePackageCallable();
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Updates a [package][google.android.overtheair.v1.Package]. Returns `INVALID_ARGUMENT` if any
   * field other than title is modified.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatPackagePathName("[DEPLOYMENT]", "[PACKAGE_PATH]");
   *   com.google.android.overtheair.v1.Package packageObj = com.google.android.overtheair.v1.Package.newBuilder().build();
   *   VersionTag versionTag = VersionTag.newBuilder().build();
   *   com.google.android.overtheair.v1.Package response = overTheAirServiceClient.updatePackage(formattedName, packageObj, versionTag);
   * }
   * </code></pre>
   *
   * @param name Relative resource name. For example: "deployments/deployment_x/packages/package_a"
   * @param packageObj Required. The [Package][google.android.overtheair.v1.Package] to update.
   * @param versionTag Required. The package to be updated should match this version. If another
   *     client updates the existing config concurrently, the update operation will fail.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final com.google.android.overtheair.v1.Package updatePackage(
      String name, com.google.android.overtheair.v1.Package packageObj, VersionTag versionTag) {
    PACKAGE_PATH_PATH_TEMPLATE.validate(name, "updatePackage");
    UpdatePackageRequest request =
        UpdatePackageRequest.newBuilder()
            .setName(name)
            .setPackageObj(packageObj)
            .setVersionTag(versionTag)
            .build();
    return updatePackage(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Updates a [package][google.android.overtheair.v1.Package]. Returns `INVALID_ARGUMENT` if any
   * field other than title is modified.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatPackagePathName("[DEPLOYMENT]", "[PACKAGE_PATH]");
   *   com.google.android.overtheair.v1.Package packageObj = com.google.android.overtheair.v1.Package.newBuilder().build();
   *   VersionTag versionTag = VersionTag.newBuilder().build();
   *   UpdatePackageRequest request = UpdatePackageRequest.newBuilder()
   *     .setName(formattedName)
   *     .setPackageObj(packageObj)
   *     .setVersionTag(versionTag)
   *     .build();
   *   com.google.android.overtheair.v1.Package response = overTheAirServiceClient.updatePackage(request);
   * }
   * </code></pre>
   *
   * @param request The request object containing all of the parameters for the API call.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final com.google.android.overtheair.v1.Package updatePackage(
      UpdatePackageRequest request) {
    return updatePackageCallable().call(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Updates a [package][google.android.overtheair.v1.Package]. Returns `INVALID_ARGUMENT` if any
   * field other than title is modified.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatPackagePathName("[DEPLOYMENT]", "[PACKAGE_PATH]");
   *   com.google.android.overtheair.v1.Package packageObj = com.google.android.overtheair.v1.Package.newBuilder().build();
   *   VersionTag versionTag = VersionTag.newBuilder().build();
   *   UpdatePackageRequest request = UpdatePackageRequest.newBuilder()
   *     .setName(formattedName)
   *     .setPackageObj(packageObj)
   *     .setVersionTag(versionTag)
   *     .build();
   *   ApiFuture&lt;com.google.android.overtheair.v1.Package&gt; future = overTheAirServiceClient.updatePackageCallable().futureCall(request);
   *   // Do something
   *   com.google.android.overtheair.v1.Package response = future.get();
   * }
   * </code></pre>
   */
  public final UnaryCallable<UpdatePackageRequest, com.google.android.overtheair.v1.Package>
      updatePackageCallable() {
    return stub.updatePackageCallable();
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Creates a config, and returns the new [Config][google.android.overtheair.v1.Config].
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatDeploymentName("[DEPLOYMENT]");
   *   Config config = Config.newBuilder().build();
   *   Config response = overTheAirServiceClient.createConfig(formattedName, config);
   * }
   * </code></pre>
   *
   * @param name Relative resource name. For example: "deployments/deployment_x"
   * @param config Required. The [Config][google.android.overtheair.v1.Config] to create.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final Config createConfig(String name, Config config) {
    DEPLOYMENT_PATH_TEMPLATE.validate(name, "createConfig");
    CreateConfigRequest request =
        CreateConfigRequest.newBuilder().setName(name).setConfig(config).build();
    return createConfig(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Creates a config, and returns the new [Config][google.android.overtheair.v1.Config].
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatDeploymentName("[DEPLOYMENT]");
   *   Config config = Config.newBuilder().build();
   *   CreateConfigRequest request = CreateConfigRequest.newBuilder()
   *     .setName(formattedName)
   *     .setConfig(config)
   *     .build();
   *   Config response = overTheAirServiceClient.createConfig(request);
   * }
   * </code></pre>
   *
   * @param request The request object containing all of the parameters for the API call.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final Config createConfig(CreateConfigRequest request) {
    return createConfigCallable().call(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Creates a config, and returns the new [Config][google.android.overtheair.v1.Config].
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatDeploymentName("[DEPLOYMENT]");
   *   Config config = Config.newBuilder().build();
   *   CreateConfigRequest request = CreateConfigRequest.newBuilder()
   *     .setName(formattedName)
   *     .setConfig(config)
   *     .build();
   *   ApiFuture&lt;Config&gt; future = overTheAirServiceClient.createConfigCallable().futureCall(request);
   *   // Do something
   *   Config response = future.get();
   * }
   * </code></pre>
   */
  public final UnaryCallable<CreateConfigRequest, Config> createConfigCallable() {
    return stub.createConfigCallable();
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Gets a [config][google.android.overtheair.v1.Config] view. Returns `NOT_FOUND` if the matching
   * path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatConfigName("[DEPLOYMENT]", "[CONFIG]");
   *   View view = View.BASIC;
   *   Config response = overTheAirServiceClient.getConfig(formattedName, view);
   * }
   * </code></pre>
   *
   * @param name Relative resource name. For example: "deployments/deployment_x/configs/config_a"
   * @param view Optional. Specify whether basic or detailed information of the config should be
   *     returned.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final Config getConfig(String name, View view) {
    CONFIG_PATH_TEMPLATE.validate(name, "getConfig");
    GetConfigRequest request = GetConfigRequest.newBuilder().setName(name).setView(view).build();
    return getConfig(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Gets a [config][google.android.overtheair.v1.Config] view. Returns `NOT_FOUND` if the matching
   * path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatConfigName("[DEPLOYMENT]", "[CONFIG]");
   *   GetConfigRequest request = GetConfigRequest.newBuilder()
   *     .setName(formattedName)
   *     .build();
   *   Config response = overTheAirServiceClient.getConfig(request);
   * }
   * </code></pre>
   *
   * @param request The request object containing all of the parameters for the API call.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final Config getConfig(GetConfigRequest request) {
    return getConfigCallable().call(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Gets a [config][google.android.overtheair.v1.Config] view. Returns `NOT_FOUND` if the matching
   * path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatConfigName("[DEPLOYMENT]", "[CONFIG]");
   *   GetConfigRequest request = GetConfigRequest.newBuilder()
   *     .setName(formattedName)
   *     .build();
   *   ApiFuture&lt;Config&gt; future = overTheAirServiceClient.getConfigCallable().futureCall(request);
   *   // Do something
   *   Config response = future.get();
   * }
   * </code></pre>
   */
  public final UnaryCallable<GetConfigRequest, Config> getConfigCallable() {
    return stub.getConfigCallable();
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Lists [configs][google.android.overtheair.v1.Config] of a deployment. Returns `NOT_FOUND` if
   * the matching path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatDeploymentName("[DEPLOYMENT]");
   *   View view = View.BASIC;
   *   ListConfigsResponse response = overTheAirServiceClient.listConfigs(formattedName, view);
   * }
   * </code></pre>
   *
   * @param name Relative resource name. For example: "deployments/deployment_x"
   * @param view Optional. Specify whether basic or detailed information of each config should be
   *     returned.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final ListConfigsResponse listConfigs(String name, View view) {
    DEPLOYMENT_PATH_TEMPLATE.validate(name, "listConfigs");
    ListConfigsRequest request =
        ListConfigsRequest.newBuilder().setName(name).setView(view).build();
    return listConfigs(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Lists [configs][google.android.overtheair.v1.Config] of a deployment. Returns `NOT_FOUND` if
   * the matching path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatDeploymentName("[DEPLOYMENT]");
   *   ListConfigsRequest request = ListConfigsRequest.newBuilder()
   *     .setName(formattedName)
   *     .build();
   *   ListConfigsResponse response = overTheAirServiceClient.listConfigs(request);
   * }
   * </code></pre>
   *
   * @param request The request object containing all of the parameters for the API call.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final ListConfigsResponse listConfigs(ListConfigsRequest request) {
    return listConfigsCallable().call(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Lists [configs][google.android.overtheair.v1.Config] of a deployment. Returns `NOT_FOUND` if
   * the matching path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatDeploymentName("[DEPLOYMENT]");
   *   ListConfigsRequest request = ListConfigsRequest.newBuilder()
   *     .setName(formattedName)
   *     .build();
   *   ApiFuture&lt;ListConfigsResponse&gt; future = overTheAirServiceClient.listConfigsCallable().futureCall(request);
   *   // Do something
   *   ListConfigsResponse response = future.get();
   * }
   * </code></pre>
   */
  public final UnaryCallable<ListConfigsRequest, ListConfigsResponse> listConfigsCallable() {
    return stub.listConfigsCallable();
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Deletes a [config][google.android.overtheair.v1.Config]. Returns `NOT_FOUND` if the matching
   * path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatConfigName("[DEPLOYMENT]", "[CONFIG]");
   *   DeleteConfigResponse response = overTheAirServiceClient.deleteConfig(formattedName);
   * }
   * </code></pre>
   *
   * @param name Relative resource name. For example: "deployments/deployment_x/configs/config_a"
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final DeleteConfigResponse deleteConfig(String name) {
    CONFIG_PATH_TEMPLATE.validate(name, "deleteConfig");
    DeleteConfigRequest request = DeleteConfigRequest.newBuilder().setName(name).build();
    return deleteConfig(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Deletes a [config][google.android.overtheair.v1.Config]. Returns `NOT_FOUND` if the matching
   * path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatConfigName("[DEPLOYMENT]", "[CONFIG]");
   *   DeleteConfigRequest request = DeleteConfigRequest.newBuilder()
   *     .setName(formattedName)
   *     .build();
   *   DeleteConfigResponse response = overTheAirServiceClient.deleteConfig(request);
   * }
   * </code></pre>
   *
   * @param request The request object containing all of the parameters for the API call.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final DeleteConfigResponse deleteConfig(DeleteConfigRequest request) {
    return deleteConfigCallable().call(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Deletes a [config][google.android.overtheair.v1.Config]. Returns `NOT_FOUND` if the matching
   * path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatConfigName("[DEPLOYMENT]", "[CONFIG]");
   *   DeleteConfigRequest request = DeleteConfigRequest.newBuilder()
   *     .setName(formattedName)
   *     .build();
   *   ApiFuture&lt;DeleteConfigResponse&gt; future = overTheAirServiceClient.deleteConfigCallable().futureCall(request);
   *   // Do something
   *   DeleteConfigResponse response = future.get();
   * }
   * </code></pre>
   */
  public final UnaryCallable<DeleteConfigRequest, DeleteConfigResponse> deleteConfigCallable() {
    return stub.deleteConfigCallable();
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Updates a [config][google.android.overtheair.v1.Config]. Returns `INVALID_ARGUMENT` if the name
   * of the config is non-empty and does not equal the existing name.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatConfigName("[DEPLOYMENT]", "[CONFIG]");
   *   Config config = Config.newBuilder().build();
   *   VersionTag versionTag = VersionTag.newBuilder().build();
   *   Config response = overTheAirServiceClient.updateConfig(formattedName, config, versionTag);
   * }
   * </code></pre>
   *
   * @param name Relative resource name. For example: "deployments/deployment_x/configs/config_a"
   * @param config The [Config][google.android.overtheair.v1.Config] to update.
   * @param versionTag Required. The config to be updated should match this version. If another
   *     client updates the existing config concurrently, the update operation will fail.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final Config updateConfig(String name, Config config, VersionTag versionTag) {
    CONFIG_PATH_TEMPLATE.validate(name, "updateConfig");
    UpdateConfigRequest request =
        UpdateConfigRequest.newBuilder()
            .setName(name)
            .setConfig(config)
            .setVersionTag(versionTag)
            .build();
    return updateConfig(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Updates a [config][google.android.overtheair.v1.Config]. Returns `INVALID_ARGUMENT` if the name
   * of the config is non-empty and does not equal the existing name.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatConfigName("[DEPLOYMENT]", "[CONFIG]");
   *   Config config = Config.newBuilder().build();
   *   VersionTag versionTag = VersionTag.newBuilder().build();
   *   UpdateConfigRequest request = UpdateConfigRequest.newBuilder()
   *     .setName(formattedName)
   *     .setConfig(config)
   *     .setVersionTag(versionTag)
   *     .build();
   *   Config response = overTheAirServiceClient.updateConfig(request);
   * }
   * </code></pre>
   *
   * @param request The request object containing all of the parameters for the API call.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final Config updateConfig(UpdateConfigRequest request) {
    return updateConfigCallable().call(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Updates a [config][google.android.overtheair.v1.Config]. Returns `INVALID_ARGUMENT` if the name
   * of the config is non-empty and does not equal the existing name.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatConfigName("[DEPLOYMENT]", "[CONFIG]");
   *   Config config = Config.newBuilder().build();
   *   VersionTag versionTag = VersionTag.newBuilder().build();
   *   UpdateConfigRequest request = UpdateConfigRequest.newBuilder()
   *     .setName(formattedName)
   *     .setConfig(config)
   *     .setVersionTag(versionTag)
   *     .build();
   *   ApiFuture&lt;Config&gt; future = overTheAirServiceClient.updateConfigCallable().futureCall(request);
   *   // Do something
   *   Config response = future.get();
   * }
   * </code></pre>
   */
  public final UnaryCallable<UpdateConfigRequest, Config> updateConfigCallable() {
    return stub.updateConfigCallable();
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Creates a group, and returns the new [Group][google.android.overtheair.v1.Group].
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatDeploymentName("[DEPLOYMENT]");
   *   Group group = Group.newBuilder().build();
   *   Group response = overTheAirServiceClient.createGroup(formattedName, group);
   * }
   * </code></pre>
   *
   * @param name Relative resource name. For example: "deployments/deployment_x"
   * @param group Required. The [Group][google.android.overtheair.v1.Group] to create.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final Group createGroup(String name, Group group) {
    DEPLOYMENT_PATH_TEMPLATE.validate(name, "createGroup");
    CreateGroupRequest request =
        CreateGroupRequest.newBuilder().setName(name).setGroup(group).build();
    return createGroup(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Creates a group, and returns the new [Group][google.android.overtheair.v1.Group].
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatDeploymentName("[DEPLOYMENT]");
   *   Group group = Group.newBuilder().build();
   *   CreateGroupRequest request = CreateGroupRequest.newBuilder()
   *     .setName(formattedName)
   *     .setGroup(group)
   *     .build();
   *   Group response = overTheAirServiceClient.createGroup(request);
   * }
   * </code></pre>
   *
   * @param request The request object containing all of the parameters for the API call.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final Group createGroup(CreateGroupRequest request) {
    return createGroupCallable().call(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Creates a group, and returns the new [Group][google.android.overtheair.v1.Group].
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatDeploymentName("[DEPLOYMENT]");
   *   Group group = Group.newBuilder().build();
   *   CreateGroupRequest request = CreateGroupRequest.newBuilder()
   *     .setName(formattedName)
   *     .setGroup(group)
   *     .build();
   *   ApiFuture&lt;Group&gt; future = overTheAirServiceClient.createGroupCallable().futureCall(request);
   *   // Do something
   *   Group response = future.get();
   * }
   * </code></pre>
   */
  public final UnaryCallable<CreateGroupRequest, Group> createGroupCallable() {
    return stub.createGroupCallable();
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Gets a [group][google.android.overtheair.v1.Group]. Returns `NOT_FOUND` if the group does not
   * exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatGroupName("[DEPLOYMENT]", "[GROUP]");
   *   View view = View.BASIC;
   *   Group response = overTheAirServiceClient.getGroup(formattedName, view);
   * }
   * </code></pre>
   *
   * @param name Relative resource name. For example: "deployments/deployment_x/groups/group_g"
   * @param view Optional. Specify whether basic or detailed information of the group should be
   *     returned.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final Group getGroup(String name, View view) {
    GROUP_PATH_TEMPLATE.validate(name, "getGroup");
    GetGroupRequest request = GetGroupRequest.newBuilder().setName(name).setView(view).build();
    return getGroup(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Gets a [group][google.android.overtheair.v1.Group]. Returns `NOT_FOUND` if the group does not
   * exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatGroupName("[DEPLOYMENT]", "[GROUP]");
   *   GetGroupRequest request = GetGroupRequest.newBuilder()
   *     .setName(formattedName)
   *     .build();
   *   Group response = overTheAirServiceClient.getGroup(request);
   * }
   * </code></pre>
   *
   * @param request The request object containing all of the parameters for the API call.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final Group getGroup(GetGroupRequest request) {
    return getGroupCallable().call(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Gets a [group][google.android.overtheair.v1.Group]. Returns `NOT_FOUND` if the group does not
   * exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatGroupName("[DEPLOYMENT]", "[GROUP]");
   *   GetGroupRequest request = GetGroupRequest.newBuilder()
   *     .setName(formattedName)
   *     .build();
   *   ApiFuture&lt;Group&gt; future = overTheAirServiceClient.getGroupCallable().futureCall(request);
   *   // Do something
   *   Group response = future.get();
   * }
   * </code></pre>
   */
  public final UnaryCallable<GetGroupRequest, Group> getGroupCallable() {
    return stub.getGroupCallable();
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Lists [groups][google.android.overtheair.v1.Group] of a deployment. Returns `NOT_FOUND` if the
   * matching path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatDeploymentName("[DEPLOYMENT]");
   *   View view = View.BASIC;
   *   ListGroupsResponse response = overTheAirServiceClient.listGroups(formattedName, view);
   * }
   * </code></pre>
   *
   * @param name Relative resource name. For example: "deployments/deployment_x"
   * @param view Optional. Specify whether basic or detailed information of each group should be
   *     returned.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final ListGroupsResponse listGroups(String name, View view) {
    DEPLOYMENT_PATH_TEMPLATE.validate(name, "listGroups");
    ListGroupsRequest request = ListGroupsRequest.newBuilder().setName(name).setView(view).build();
    return listGroups(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Lists [groups][google.android.overtheair.v1.Group] of a deployment. Returns `NOT_FOUND` if the
   * matching path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatDeploymentName("[DEPLOYMENT]");
   *   ListGroupsRequest request = ListGroupsRequest.newBuilder()
   *     .setName(formattedName)
   *     .build();
   *   ListGroupsResponse response = overTheAirServiceClient.listGroups(request);
   * }
   * </code></pre>
   *
   * @param request The request object containing all of the parameters for the API call.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final ListGroupsResponse listGroups(ListGroupsRequest request) {
    return listGroupsCallable().call(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Lists [groups][google.android.overtheair.v1.Group] of a deployment. Returns `NOT_FOUND` if the
   * matching path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatDeploymentName("[DEPLOYMENT]");
   *   ListGroupsRequest request = ListGroupsRequest.newBuilder()
   *     .setName(formattedName)
   *     .build();
   *   ApiFuture&lt;ListGroupsResponse&gt; future = overTheAirServiceClient.listGroupsCallable().futureCall(request);
   *   // Do something
   *   ListGroupsResponse response = future.get();
   * }
   * </code></pre>
   */
  public final UnaryCallable<ListGroupsRequest, ListGroupsResponse> listGroupsCallable() {
    return stub.listGroupsCallable();
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Deletes a [group][google.android.overtheair.v1.Group]. Returns `NOT_FOUND` if the group does
   * not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatGroupName("[DEPLOYMENT]", "[GROUP]");
   *   DeleteGroupResponse response = overTheAirServiceClient.deleteGroup(formattedName);
   * }
   * </code></pre>
   *
   * @param name Relative resource name. For example: "deployments/deployment_x/groups/group_g"
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final DeleteGroupResponse deleteGroup(String name) {
    GROUP_PATH_TEMPLATE.validate(name, "deleteGroup");
    DeleteGroupRequest request = DeleteGroupRequest.newBuilder().setName(name).build();
    return deleteGroup(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Deletes a [group][google.android.overtheair.v1.Group]. Returns `NOT_FOUND` if the group does
   * not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatGroupName("[DEPLOYMENT]", "[GROUP]");
   *   DeleteGroupRequest request = DeleteGroupRequest.newBuilder()
   *     .setName(formattedName)
   *     .build();
   *   DeleteGroupResponse response = overTheAirServiceClient.deleteGroup(request);
   * }
   * </code></pre>
   *
   * @param request The request object containing all of the parameters for the API call.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final DeleteGroupResponse deleteGroup(DeleteGroupRequest request) {
    return deleteGroupCallable().call(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Deletes a [group][google.android.overtheair.v1.Group]. Returns `NOT_FOUND` if the group does
   * not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatGroupName("[DEPLOYMENT]", "[GROUP]");
   *   DeleteGroupRequest request = DeleteGroupRequest.newBuilder()
   *     .setName(formattedName)
   *     .build();
   *   ApiFuture&lt;DeleteGroupResponse&gt; future = overTheAirServiceClient.deleteGroupCallable().futureCall(request);
   *   // Do something
   *   DeleteGroupResponse response = future.get();
   * }
   * </code></pre>
   */
  public final UnaryCallable<DeleteGroupRequest, DeleteGroupResponse> deleteGroupCallable() {
    return stub.deleteGroupCallable();
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Updates a [group][google.android.overtheair.v1.Group]. Returns INVALID_ARGUMENT if the name of
   * the group is non-empty and does not equal the existing name.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatGroupName("[DEPLOYMENT]", "[GROUP]");
   *   Group group = Group.newBuilder().build();
   *   VersionTag versionTag = VersionTag.newBuilder().build();
   *   Group response = overTheAirServiceClient.updateGroup(formattedName, group, versionTag);
   * }
   * </code></pre>
   *
   * @param name Relative resource name. For example: "deployments/deployment_x/groups/group_g"
   * @param group Required. The [Group][google.android.overtheair.v1.Group] to update.
   * @param versionTag Required. The group to be updated should match this version. If another
   *     client updates the existing config concurrently, the update operation will fail.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final Group updateGroup(String name, Group group, VersionTag versionTag) {
    GROUP_PATH_TEMPLATE.validate(name, "updateGroup");
    UpdateGroupRequest request =
        UpdateGroupRequest.newBuilder()
            .setName(name)
            .setGroup(group)
            .setVersionTag(versionTag)
            .build();
    return updateGroup(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Updates a [group][google.android.overtheair.v1.Group]. Returns INVALID_ARGUMENT if the name of
   * the group is non-empty and does not equal the existing name.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatGroupName("[DEPLOYMENT]", "[GROUP]");
   *   Group group = Group.newBuilder().build();
   *   VersionTag versionTag = VersionTag.newBuilder().build();
   *   UpdateGroupRequest request = UpdateGroupRequest.newBuilder()
   *     .setName(formattedName)
   *     .setGroup(group)
   *     .setVersionTag(versionTag)
   *     .build();
   *   Group response = overTheAirServiceClient.updateGroup(request);
   * }
   * </code></pre>
   *
   * @param request The request object containing all of the parameters for the API call.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final Group updateGroup(UpdateGroupRequest request) {
    return updateGroupCallable().call(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Updates a [group][google.android.overtheair.v1.Group]. Returns INVALID_ARGUMENT if the name of
   * the group is non-empty and does not equal the existing name.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatGroupName("[DEPLOYMENT]", "[GROUP]");
   *   Group group = Group.newBuilder().build();
   *   VersionTag versionTag = VersionTag.newBuilder().build();
   *   UpdateGroupRequest request = UpdateGroupRequest.newBuilder()
   *     .setName(formattedName)
   *     .setGroup(group)
   *     .setVersionTag(versionTag)
   *     .build();
   *   ApiFuture&lt;Group&gt; future = overTheAirServiceClient.updateGroupCallable().futureCall(request);
   *   // Do something
   *   Group response = future.get();
   * }
   * </code></pre>
   */
  public final UnaryCallable<UpdateGroupRequest, Group> updateGroupCallable() {
    return stub.updateGroupCallable();
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Disables the defined [groups][google.android.overtheair.v1.Group]. Returns `NOT_FOUND` if the
   * matching path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatDeploymentName("[DEPLOYMENT]");
   *   List&lt;String&gt; groupNames = new ArrayList&lt;&gt;();
   *   DisableGroupsResponse response = overTheAirServiceClient.disableGroups(formattedName, groupNames);
   * }
   * </code></pre>
   *
   * @param name Relative resource name. For example: "deployments/deployment_x/groups"
   * @param groupNames Required. The groups that should be disabled.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final DisableGroupsResponse disableGroups(String name, List<String> groupNames) {
    DEPLOYMENT_PATH_TEMPLATE.validate(name, "disableGroups");
    DisableGroupsRequest request =
        DisableGroupsRequest.newBuilder().setName(name).addAllGroupNames(groupNames).build();
    return disableGroups(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Disables the defined [groups][google.android.overtheair.v1.Group]. Returns `NOT_FOUND` if the
   * matching path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatDeploymentName("[DEPLOYMENT]");
   *   List&lt;String&gt; groupNames = new ArrayList&lt;&gt;();
   *   DisableGroupsRequest request = DisableGroupsRequest.newBuilder()
   *     .setName(formattedName)
   *     .addAllGroupNames(groupNames)
   *     .build();
   *   DisableGroupsResponse response = overTheAirServiceClient.disableGroups(request);
   * }
   * </code></pre>
   *
   * @param request The request object containing all of the parameters for the API call.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final DisableGroupsResponse disableGroups(DisableGroupsRequest request) {
    return disableGroupsCallable().call(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Disables the defined [groups][google.android.overtheair.v1.Group]. Returns `NOT_FOUND` if the
   * matching path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatDeploymentName("[DEPLOYMENT]");
   *   List&lt;String&gt; groupNames = new ArrayList&lt;&gt;();
   *   DisableGroupsRequest request = DisableGroupsRequest.newBuilder()
   *     .setName(formattedName)
   *     .addAllGroupNames(groupNames)
   *     .build();
   *   ApiFuture&lt;DisableGroupsResponse&gt; future = overTheAirServiceClient.disableGroupsCallable().futureCall(request);
   *   // Do something
   *   DisableGroupsResponse response = future.get();
   * }
   * </code></pre>
   */
  public final UnaryCallable<DisableGroupsRequest, DisableGroupsResponse> disableGroupsCallable() {
    return stub.disableGroupsCallable();
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Enables the defined [groups][google.android.overtheair.v1.Group]. Returns `NOT_FOUND` if the
   * matching path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatDeploymentName("[DEPLOYMENT]");
   *   List&lt;String&gt; groupNames = new ArrayList&lt;&gt;();
   *   EnableGroupsResponse response = overTheAirServiceClient.enableGroups(formattedName, groupNames);
   * }
   * </code></pre>
   *
   * @param name Relative resource name. For example: "deployments/deployment_x/groups"
   * @param groupNames Required. The groups should be enabled.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final EnableGroupsResponse enableGroups(String name, List<String> groupNames) {
    DEPLOYMENT_PATH_TEMPLATE.validate(name, "enableGroups");
    EnableGroupsRequest request =
        EnableGroupsRequest.newBuilder().setName(name).addAllGroupNames(groupNames).build();
    return enableGroups(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Enables the defined [groups][google.android.overtheair.v1.Group]. Returns `NOT_FOUND` if the
   * matching path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatDeploymentName("[DEPLOYMENT]");
   *   List&lt;String&gt; groupNames = new ArrayList&lt;&gt;();
   *   EnableGroupsRequest request = EnableGroupsRequest.newBuilder()
   *     .setName(formattedName)
   *     .addAllGroupNames(groupNames)
   *     .build();
   *   EnableGroupsResponse response = overTheAirServiceClient.enableGroups(request);
   * }
   * </code></pre>
   *
   * @param request The request object containing all of the parameters for the API call.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final EnableGroupsResponse enableGroups(EnableGroupsRequest request) {
    return enableGroupsCallable().call(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Enables the defined [groups][google.android.overtheair.v1.Group]. Returns `NOT_FOUND` if the
   * matching path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatDeploymentName("[DEPLOYMENT]");
   *   List&lt;String&gt; groupNames = new ArrayList&lt;&gt;();
   *   EnableGroupsRequest request = EnableGroupsRequest.newBuilder()
   *     .setName(formattedName)
   *     .addAllGroupNames(groupNames)
   *     .build();
   *   ApiFuture&lt;EnableGroupsResponse&gt; future = overTheAirServiceClient.enableGroupsCallable().futureCall(request);
   *   // Do something
   *   EnableGroupsResponse response = future.get();
   * }
   * </code></pre>
   */
  public final UnaryCallable<EnableGroupsRequest, EnableGroupsResponse> enableGroupsCallable() {
    return stub.enableGroupsCallable();
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Creates a group member and returns the new
   * [GroupMember][google.android.overtheair.v1.GroupMember]. If the device is already under other
   * groups, it will be removed from those groups and added to the group that's specified in the
   * request.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatGroupName("[DEPLOYMENT]", "[GROUP]");
   *   GroupMember groupMember = GroupMember.newBuilder().build();
   *   GroupMember response = overTheAirServiceClient.createGroupMember(formattedName, groupMember);
   * }
   * </code></pre>
   *
   * @param name Relative resource name. For example:
   *     "deployments/deployment_x/groups/group_g/members"
   * @param groupMember Required. The [group member][google.android.overtheair.v1.GroupMember] to
   *     create.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final GroupMember createGroupMember(String name, GroupMember groupMember) {
    GROUP_PATH_TEMPLATE.validate(name, "createGroupMember");
    CreateGroupMemberRequest request =
        CreateGroupMemberRequest.newBuilder().setName(name).setGroupMember(groupMember).build();
    return createGroupMember(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Creates a group member and returns the new
   * [GroupMember][google.android.overtheair.v1.GroupMember]. If the device is already under other
   * groups, it will be removed from those groups and added to the group that's specified in the
   * request.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatGroupName("[DEPLOYMENT]", "[GROUP]");
   *   GroupMember groupMember = GroupMember.newBuilder().build();
   *   CreateGroupMemberRequest request = CreateGroupMemberRequest.newBuilder()
   *     .setName(formattedName)
   *     .setGroupMember(groupMember)
   *     .build();
   *   GroupMember response = overTheAirServiceClient.createGroupMember(request);
   * }
   * </code></pre>
   *
   * @param request The request object containing all of the parameters for the API call.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final GroupMember createGroupMember(CreateGroupMemberRequest request) {
    return createGroupMemberCallable().call(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Creates a group member and returns the new
   * [GroupMember][google.android.overtheair.v1.GroupMember]. If the device is already under other
   * groups, it will be removed from those groups and added to the group that's specified in the
   * request.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatGroupName("[DEPLOYMENT]", "[GROUP]");
   *   GroupMember groupMember = GroupMember.newBuilder().build();
   *   CreateGroupMemberRequest request = CreateGroupMemberRequest.newBuilder()
   *     .setName(formattedName)
   *     .setGroupMember(groupMember)
   *     .build();
   *   ApiFuture&lt;GroupMember&gt; future = overTheAirServiceClient.createGroupMemberCallable().futureCall(request);
   *   // Do something
   *   GroupMember response = future.get();
   * }
   * </code></pre>
   */
  public final UnaryCallable<CreateGroupMemberRequest, GroupMember> createGroupMemberCallable() {
    return stub.createGroupMemberCallable();
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Creates group members in a specific group. If any device is in other groups, it will be removed
   * from those groups and added to the group that's specified in the request.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatGroupName("[DEPLOYMENT]", "[GROUP]");
   *   List&lt;GroupMember&gt; groupMembers = new ArrayList&lt;&gt;();
   *   CreateGroupMembersResponse response = overTheAirServiceClient.createGroupMembers(formattedName, groupMembers);
   * }
   * </code></pre>
   *
   * @param name Relative resource name. For example:
   *     "deployments/deployment_x/groups/group_g/members"
   * @param groupMembers Required. The list of [group
   *     member][google.android.overtheair.v1.GroupMember]s to create. This list is currently
   *     limited to no more than 1,000 members. If you need to create more than that, please send
   *     multiple requests.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final CreateGroupMembersResponse createGroupMembers(
      String name, List<GroupMember> groupMembers) {
    GROUP_PATH_TEMPLATE.validate(name, "createGroupMembers");
    CreateGroupMembersRequest request =
        CreateGroupMembersRequest.newBuilder()
            .setName(name)
            .addAllGroupMembers(groupMembers)
            .build();
    return createGroupMembers(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Creates group members in a specific group. If any device is in other groups, it will be removed
   * from those groups and added to the group that's specified in the request.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatGroupName("[DEPLOYMENT]", "[GROUP]");
   *   List&lt;GroupMember&gt; groupMembers = new ArrayList&lt;&gt;();
   *   CreateGroupMembersRequest request = CreateGroupMembersRequest.newBuilder()
   *     .setName(formattedName)
   *     .addAllGroupMembers(groupMembers)
   *     .build();
   *   CreateGroupMembersResponse response = overTheAirServiceClient.createGroupMembers(request);
   * }
   * </code></pre>
   *
   * @param request The request object containing all of the parameters for the API call.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final CreateGroupMembersResponse createGroupMembers(CreateGroupMembersRequest request) {
    return createGroupMembersCallable().call(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Creates group members in a specific group. If any device is in other groups, it will be removed
   * from those groups and added to the group that's specified in the request.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatGroupName("[DEPLOYMENT]", "[GROUP]");
   *   List&lt;GroupMember&gt; groupMembers = new ArrayList&lt;&gt;();
   *   CreateGroupMembersRequest request = CreateGroupMembersRequest.newBuilder()
   *     .setName(formattedName)
   *     .addAllGroupMembers(groupMembers)
   *     .build();
   *   ApiFuture&lt;CreateGroupMembersResponse&gt; future = overTheAirServiceClient.createGroupMembersCallable().futureCall(request);
   *   // Do something
   *   CreateGroupMembersResponse response = future.get();
   * }
   * </code></pre>
   */
  public final UnaryCallable<CreateGroupMembersRequest, CreateGroupMembersResponse>
      createGroupMembersCallable() {
    return stub.createGroupMembersCallable();
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Gets a [group member][google.android.overtheair.v1.GroupMember]. Returns `NOT_FOUND` if the
   * matching path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatMemberName("[DEPLOYMENT]", "[GROUP]", "[MEMBER]");
   *   View view = View.BASIC;
   *   GroupMember response = overTheAirServiceClient.getGroupMember(formattedName, view);
   * }
   * </code></pre>
   *
   * @param name Relative resource name. For example:
   *     "deployments/deployment_x/groups/group_g/members/member_m"
   * @param view Optional. Specify whether basic or detailed information of the group member should
   *     be returned.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final GroupMember getGroupMember(String name, View view) {
    MEMBER_PATH_TEMPLATE.validate(name, "getGroupMember");
    GetGroupMemberRequest request =
        GetGroupMemberRequest.newBuilder().setName(name).setView(view).build();
    return getGroupMember(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Gets a [group member][google.android.overtheair.v1.GroupMember]. Returns `NOT_FOUND` if the
   * matching path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatMemberName("[DEPLOYMENT]", "[GROUP]", "[MEMBER]");
   *   GetGroupMemberRequest request = GetGroupMemberRequest.newBuilder()
   *     .setName(formattedName)
   *     .build();
   *   GroupMember response = overTheAirServiceClient.getGroupMember(request);
   * }
   * </code></pre>
   *
   * @param request The request object containing all of the parameters for the API call.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final GroupMember getGroupMember(GetGroupMemberRequest request) {
    return getGroupMemberCallable().call(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Gets a [group member][google.android.overtheair.v1.GroupMember]. Returns `NOT_FOUND` if the
   * matching path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatMemberName("[DEPLOYMENT]", "[GROUP]", "[MEMBER]");
   *   GetGroupMemberRequest request = GetGroupMemberRequest.newBuilder()
   *     .setName(formattedName)
   *     .build();
   *   ApiFuture&lt;GroupMember&gt; future = overTheAirServiceClient.getGroupMemberCallable().futureCall(request);
   *   // Do something
   *   GroupMember response = future.get();
   * }
   * </code></pre>
   */
  public final UnaryCallable<GetGroupMemberRequest, GroupMember> getGroupMemberCallable() {
    return stub.getGroupMemberCallable();
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Lists [group members][google.android.overtheair.v1.GroupMember]. Returns `NOT_FOUND` if the
   * matching path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatGroupName("[DEPLOYMENT]", "[GROUP]");
   *   ListGroupMembersResponse response = overTheAirServiceClient.listGroupMembers(formattedName);
   * }
   * </code></pre>
   *
   * @param name Relative resource name. For example:
   *     "deployments/deployment_x/groups/group_g/members"
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final ListGroupMembersResponse listGroupMembers(String name) {
    GROUP_PATH_TEMPLATE.validate(name, "listGroupMembers");
    ListGroupMembersRequest request = ListGroupMembersRequest.newBuilder().setName(name).build();
    return listGroupMembers(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Lists [group members][google.android.overtheair.v1.GroupMember]. Returns `NOT_FOUND` if the
   * matching path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatGroupName("[DEPLOYMENT]", "[GROUP]");
   *   ListGroupMembersRequest request = ListGroupMembersRequest.newBuilder()
   *     .setName(formattedName)
   *     .build();
   *   ListGroupMembersResponse response = overTheAirServiceClient.listGroupMembers(request);
   * }
   * </code></pre>
   *
   * @param request The request object containing all of the parameters for the API call.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final ListGroupMembersResponse listGroupMembers(ListGroupMembersRequest request) {
    return listGroupMembersCallable().call(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Lists [group members][google.android.overtheair.v1.GroupMember]. Returns `NOT_FOUND` if the
   * matching path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatGroupName("[DEPLOYMENT]", "[GROUP]");
   *   ListGroupMembersRequest request = ListGroupMembersRequest.newBuilder()
   *     .setName(formattedName)
   *     .build();
   *   ApiFuture&lt;ListGroupMembersResponse&gt; future = overTheAirServiceClient.listGroupMembersCallable().futureCall(request);
   *   // Do something
   *   ListGroupMembersResponse response = future.get();
   * }
   * </code></pre>
   */
  public final UnaryCallable<ListGroupMembersRequest, ListGroupMembersResponse>
      listGroupMembersCallable() {
    return stub.listGroupMembersCallable();
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Deletes a [group member][google.android.overtheair.v1.GroupMember]. Returns `NOT_FOUND` if the
   * matching path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatMemberName("[DEPLOYMENT]", "[GROUP]", "[MEMBER]");
   *   DeleteGroupMemberResponse response = overTheAirServiceClient.deleteGroupMember(formattedName);
   * }
   * </code></pre>
   *
   * @param name Relative resource name. For example:
   *     "deployments/deployment_x/groups/group_g/members/member_m"
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final DeleteGroupMemberResponse deleteGroupMember(String name) {
    MEMBER_PATH_TEMPLATE.validate(name, "deleteGroupMember");
    DeleteGroupMemberRequest request = DeleteGroupMemberRequest.newBuilder().setName(name).build();
    return deleteGroupMember(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Deletes a [group member][google.android.overtheair.v1.GroupMember]. Returns `NOT_FOUND` if the
   * matching path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatMemberName("[DEPLOYMENT]", "[GROUP]", "[MEMBER]");
   *   DeleteGroupMemberRequest request = DeleteGroupMemberRequest.newBuilder()
   *     .setName(formattedName)
   *     .build();
   *   DeleteGroupMemberResponse response = overTheAirServiceClient.deleteGroupMember(request);
   * }
   * </code></pre>
   *
   * @param request The request object containing all of the parameters for the API call.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final DeleteGroupMemberResponse deleteGroupMember(DeleteGroupMemberRequest request) {
    return deleteGroupMemberCallable().call(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Deletes a [group member][google.android.overtheair.v1.GroupMember]. Returns `NOT_FOUND` if the
   * matching path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatMemberName("[DEPLOYMENT]", "[GROUP]", "[MEMBER]");
   *   DeleteGroupMemberRequest request = DeleteGroupMemberRequest.newBuilder()
   *     .setName(formattedName)
   *     .build();
   *   ApiFuture&lt;DeleteGroupMemberResponse&gt; future = overTheAirServiceClient.deleteGroupMemberCallable().futureCall(request);
   *   // Do something
   *   DeleteGroupMemberResponse response = future.get();
   * }
   * </code></pre>
   */
  public final UnaryCallable<DeleteGroupMemberRequest, DeleteGroupMemberResponse>
      deleteGroupMemberCallable() {
    return stub.deleteGroupMemberCallable();
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Deletes group members in a group. If any member does not it exist, it will be ignored.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatGroupName("[DEPLOYMENT]", "[GROUP]");
   *   List&lt;GroupMember&gt; groupMembers = new ArrayList&lt;&gt;();
   *   DeleteGroupMembersResponse response = overTheAirServiceClient.deleteGroupMembers(formattedName, groupMembers);
   * }
   * </code></pre>
   *
   * @param name Relative resource name. For example:
   *     "deployments/deployment_x/groups/group_g/members"
   * @param groupMembers Required. The list of [group
   *     member][google.android.overtheair.v1.GroupMember] to delete. This list is currently limited
   *     to no more than 1,000 members. If you need to delete more than that, please send multiple
   *     requests.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final DeleteGroupMembersResponse deleteGroupMembers(
      String name, List<GroupMember> groupMembers) {
    GROUP_PATH_TEMPLATE.validate(name, "deleteGroupMembers");
    DeleteGroupMembersRequest request =
        DeleteGroupMembersRequest.newBuilder()
            .setName(name)
            .addAllGroupMembers(groupMembers)
            .build();
    return deleteGroupMembers(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Deletes group members in a group. If any member does not it exist, it will be ignored.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatGroupName("[DEPLOYMENT]", "[GROUP]");
   *   List&lt;GroupMember&gt; groupMembers = new ArrayList&lt;&gt;();
   *   DeleteGroupMembersRequest request = DeleteGroupMembersRequest.newBuilder()
   *     .setName(formattedName)
   *     .addAllGroupMembers(groupMembers)
   *     .build();
   *   DeleteGroupMembersResponse response = overTheAirServiceClient.deleteGroupMembers(request);
   * }
   * </code></pre>
   *
   * @param request The request object containing all of the parameters for the API call.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final DeleteGroupMembersResponse deleteGroupMembers(DeleteGroupMembersRequest request) {
    return deleteGroupMembersCallable().call(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Deletes group members in a group. If any member does not it exist, it will be ignored.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatGroupName("[DEPLOYMENT]", "[GROUP]");
   *   List&lt;GroupMember&gt; groupMembers = new ArrayList&lt;&gt;();
   *   DeleteGroupMembersRequest request = DeleteGroupMembersRequest.newBuilder()
   *     .setName(formattedName)
   *     .addAllGroupMembers(groupMembers)
   *     .build();
   *   ApiFuture&lt;DeleteGroupMembersResponse&gt; future = overTheAirServiceClient.deleteGroupMembersCallable().futureCall(request);
   *   // Do something
   *   DeleteGroupMembersResponse response = future.get();
   * }
   * </code></pre>
   */
  public final UnaryCallable<DeleteGroupMembersRequest, DeleteGroupMembersResponse>
      deleteGroupMembersCallable() {
    return stub.deleteGroupMembersCallable();
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Gets a [group chart][google.android.overtheair.v1.Chart]. Returns `NOT_FOUND` if the matching
   * path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatChartName("[DEPLOYMENT]", "[GROUP]", "[CHART]");
   *   ChartOptions options = ChartOptions.newBuilder().build();
   *   Chart response = overTheAirServiceClient.getGroupChart(formattedName, options);
   * }
   * </code></pre>
   *
   * @param name Relative resource name. For example:
   *     "deployments/deployment_x/groups/group_g/charts/chart_c"
   * @param options Optional. Chart Options.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final Chart getGroupChart(String name, ChartOptions options) {
    CHART_PATH_TEMPLATE.validate(name, "getGroupChart");
    GetGroupChartRequest request =
        GetGroupChartRequest.newBuilder().setName(name).setOptions(options).build();
    return getGroupChart(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Gets a [group chart][google.android.overtheair.v1.Chart]. Returns `NOT_FOUND` if the matching
   * path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatChartName("[DEPLOYMENT]", "[GROUP]", "[CHART]");
   *   GetGroupChartRequest request = GetGroupChartRequest.newBuilder()
   *     .setName(formattedName)
   *     .build();
   *   Chart response = overTheAirServiceClient.getGroupChart(request);
   * }
   * </code></pre>
   *
   * @param request The request object containing all of the parameters for the API call.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final Chart getGroupChart(GetGroupChartRequest request) {
    return getGroupChartCallable().call(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Gets a [group chart][google.android.overtheair.v1.Chart]. Returns `NOT_FOUND` if the matching
   * path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatChartName("[DEPLOYMENT]", "[GROUP]", "[CHART]");
   *   GetGroupChartRequest request = GetGroupChartRequest.newBuilder()
   *     .setName(formattedName)
   *     .build();
   *   ApiFuture&lt;Chart&gt; future = overTheAirServiceClient.getGroupChartCallable().futureCall(request);
   *   // Do something
   *   Chart response = future.get();
   * }
   * </code></pre>
   */
  public final UnaryCallable<GetGroupChartRequest, Chart> getGroupChartCallable() {
    return stub.getGroupChartCallable();
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Lists [group charts][google.android.overtheair.v1.Chart]. Returns `NOT_FOUND` if the matching
   * path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatGroupName("[DEPLOYMENT]", "[GROUP]");
   *   ChartOptions options = ChartOptions.newBuilder().build();
   *   View view = View.BASIC;
   *   ListGroupChartsResponse response = overTheAirServiceClient.listGroupCharts(formattedName, options, view);
   * }
   * </code></pre>
   *
   * @param name Relative resource name. For example: "deployments/deployment_x/groups/group_g"
   * @param options Optional. Chart Options.
   * @param view Optional. Specify whether basic or detailed information of the group chart should
   *     be returned. Basic data only includes chart names.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final ListGroupChartsResponse listGroupCharts(
      String name, ChartOptions options, View view) {
    GROUP_PATH_TEMPLATE.validate(name, "listGroupCharts");
    ListGroupChartsRequest request =
        ListGroupChartsRequest.newBuilder().setName(name).setOptions(options).setView(view).build();
    return listGroupCharts(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Lists [group charts][google.android.overtheair.v1.Chart]. Returns `NOT_FOUND` if the matching
   * path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatGroupName("[DEPLOYMENT]", "[GROUP]");
   *   ListGroupChartsRequest request = ListGroupChartsRequest.newBuilder()
   *     .setName(formattedName)
   *     .build();
   *   ListGroupChartsResponse response = overTheAirServiceClient.listGroupCharts(request);
   * }
   * </code></pre>
   *
   * @param request The request object containing all of the parameters for the API call.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final ListGroupChartsResponse listGroupCharts(ListGroupChartsRequest request) {
    return listGroupChartsCallable().call(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Lists [group charts][google.android.overtheair.v1.Chart]. Returns `NOT_FOUND` if the matching
   * path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatGroupName("[DEPLOYMENT]", "[GROUP]");
   *   ListGroupChartsRequest request = ListGroupChartsRequest.newBuilder()
   *     .setName(formattedName)
   *     .build();
   *   ApiFuture&lt;ListGroupChartsResponse&gt; future = overTheAirServiceClient.listGroupChartsCallable().futureCall(request);
   *   // Do something
   *   ListGroupChartsResponse response = future.get();
   * }
   * </code></pre>
   */
  public final UnaryCallable<ListGroupChartsRequest, ListGroupChartsResponse>
      listGroupChartsCallable() {
    return stub.listGroupChartsCallable();
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Lists configs of the group. Returns `NOT_FOUND` if the matching path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatGroupName("[DEPLOYMENT]", "[GROUP]");
   *   ListGroupConfigsResponse response = overTheAirServiceClient.listGroupConfigs(formattedName);
   * }
   * </code></pre>
   *
   * @param name Relative resource name. For example: "deployments/deployment_x/groups/group_g"
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final ListGroupConfigsResponse listGroupConfigs(String name) {
    GROUP_PATH_TEMPLATE.validate(name, "listGroupConfigs");
    ListGroupConfigsRequest request = ListGroupConfigsRequest.newBuilder().setName(name).build();
    return listGroupConfigs(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Lists configs of the group. Returns `NOT_FOUND` if the matching path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatGroupName("[DEPLOYMENT]", "[GROUP]");
   *   ListGroupConfigsRequest request = ListGroupConfigsRequest.newBuilder()
   *     .setName(formattedName)
   *     .build();
   *   ListGroupConfigsResponse response = overTheAirServiceClient.listGroupConfigs(request);
   * }
   * </code></pre>
   *
   * @param request The request object containing all of the parameters for the API call.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final ListGroupConfigsResponse listGroupConfigs(ListGroupConfigsRequest request) {
    return listGroupConfigsCallable().call(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Lists configs of the group. Returns `NOT_FOUND` if the matching path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatGroupName("[DEPLOYMENT]", "[GROUP]");
   *   ListGroupConfigsRequest request = ListGroupConfigsRequest.newBuilder()
   *     .setName(formattedName)
   *     .build();
   *   ApiFuture&lt;ListGroupConfigsResponse&gt; future = overTheAirServiceClient.listGroupConfigsCallable().futureCall(request);
   *   // Do something
   *   ListGroupConfigsResponse response = future.get();
   * }
   * </code></pre>
   */
  public final UnaryCallable<ListGroupConfigsRequest, ListGroupConfigsResponse>
      listGroupConfigsCallable() {
    return stub.listGroupConfigsCallable();
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Update the configs under a group. Returns `NOT_FOUND` if the matching group or config does not
   * exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatGroupName("[DEPLOYMENT]", "[GROUP]");
   *   List&lt;String&gt; configNames = new ArrayList&lt;&gt;();
   *   VersionTag versionTag = VersionTag.newBuilder().build();
   *   UpdateGroupConfigsResponse response = overTheAirServiceClient.updateGroupConfigs(formattedName, configNames, versionTag);
   * }
   * </code></pre>
   *
   * @param name Relative resource name. For example: "deployments/deployment_x/groups/group_g"
   * @param configNames Required. The names of the configs under the group. Important: the order of
   *     configs is important. If a device matches multiple sets of packages across configs, the
   *     last matching config wins and will be used.
   * @param versionTag Required. The group to be updated should match this version. If another
   *     client updates the existing group concurrently, the update operation will fail.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final UpdateGroupConfigsResponse updateGroupConfigs(
      String name, List<String> configNames, VersionTag versionTag) {
    GROUP_PATH_TEMPLATE.validate(name, "updateGroupConfigs");
    UpdateGroupConfigsRequest request =
        UpdateGroupConfigsRequest.newBuilder()
            .setName(name)
            .addAllConfigNames(configNames)
            .setVersionTag(versionTag)
            .build();
    return updateGroupConfigs(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Update the configs under a group. Returns `NOT_FOUND` if the matching group or config does not
   * exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatGroupName("[DEPLOYMENT]", "[GROUP]");
   *   List&lt;String&gt; configNames = new ArrayList&lt;&gt;();
   *   VersionTag versionTag = VersionTag.newBuilder().build();
   *   UpdateGroupConfigsRequest request = UpdateGroupConfigsRequest.newBuilder()
   *     .setName(formattedName)
   *     .addAllConfigNames(configNames)
   *     .setVersionTag(versionTag)
   *     .build();
   *   UpdateGroupConfigsResponse response = overTheAirServiceClient.updateGroupConfigs(request);
   * }
   * </code></pre>
   *
   * @param request The request object containing all of the parameters for the API call.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final UpdateGroupConfigsResponse updateGroupConfigs(UpdateGroupConfigsRequest request) {
    return updateGroupConfigsCallable().call(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Update the configs under a group. Returns `NOT_FOUND` if the matching group or config does not
   * exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatGroupName("[DEPLOYMENT]", "[GROUP]");
   *   List&lt;String&gt; configNames = new ArrayList&lt;&gt;();
   *   VersionTag versionTag = VersionTag.newBuilder().build();
   *   UpdateGroupConfigsRequest request = UpdateGroupConfigsRequest.newBuilder()
   *     .setName(formattedName)
   *     .addAllConfigNames(configNames)
   *     .setVersionTag(versionTag)
   *     .build();
   *   ApiFuture&lt;UpdateGroupConfigsResponse&gt; future = overTheAirServiceClient.updateGroupConfigsCallable().futureCall(request);
   *   // Do something
   *   UpdateGroupConfigsResponse response = future.get();
   * }
   * </code></pre>
   */
  public final UnaryCallable<UpdateGroupConfigsRequest, UpdateGroupConfigsResponse>
      updateGroupConfigsCallable() {
    return stub.updateGroupConfigsCallable();
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Disables all enabled groups that have the specified config. Returns `NOT_FOUND` if the matching
   * path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatConfigName("[DEPLOYMENT]", "[CONFIG]");
   *   DisableAllEnabledGroupsOfConfigResponse response = overTheAirServiceClient.disableAllEnabledGroupsOfConfig(formattedName);
   * }
   * </code></pre>
   *
   * @param name Relative resource name. For example: "deployments/deployment_x/configs/config_c"
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final DisableAllEnabledGroupsOfConfigResponse disableAllEnabledGroupsOfConfig(
      String name) {
    CONFIG_PATH_TEMPLATE.validate(name, "disableAllEnabledGroupsOfConfig");
    DisableAllEnabledGroupsOfConfigRequest request =
        DisableAllEnabledGroupsOfConfigRequest.newBuilder().setName(name).build();
    return disableAllEnabledGroupsOfConfig(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Disables all enabled groups that have the specified config. Returns `NOT_FOUND` if the matching
   * path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatConfigName("[DEPLOYMENT]", "[CONFIG]");
   *   DisableAllEnabledGroupsOfConfigRequest request = DisableAllEnabledGroupsOfConfigRequest.newBuilder()
   *     .setName(formattedName)
   *     .build();
   *   DisableAllEnabledGroupsOfConfigResponse response = overTheAirServiceClient.disableAllEnabledGroupsOfConfig(request);
   * }
   * </code></pre>
   *
   * @param request The request object containing all of the parameters for the API call.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final DisableAllEnabledGroupsOfConfigResponse disableAllEnabledGroupsOfConfig(
      DisableAllEnabledGroupsOfConfigRequest request) {
    return disableAllEnabledGroupsOfConfigCallable().call(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Disables all enabled groups that have the specified config. Returns `NOT_FOUND` if the matching
   * path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatConfigName("[DEPLOYMENT]", "[CONFIG]");
   *   DisableAllEnabledGroupsOfConfigRequest request = DisableAllEnabledGroupsOfConfigRequest.newBuilder()
   *     .setName(formattedName)
   *     .build();
   *   ApiFuture&lt;DisableAllEnabledGroupsOfConfigResponse&gt; future = overTheAirServiceClient.disableAllEnabledGroupsOfConfigCallable().futureCall(request);
   *   // Do something
   *   DisableAllEnabledGroupsOfConfigResponse response = future.get();
   * }
   * </code></pre>
   */
  public final UnaryCallable<
          DisableAllEnabledGroupsOfConfigRequest, DisableAllEnabledGroupsOfConfigResponse>
      disableAllEnabledGroupsOfConfigCallable() {
    return stub.disableAllEnabledGroupsOfConfigCallable();
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Lists packages of the config. Returns `NOT_FOUND` if the matching path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatConfigName("[DEPLOYMENT]", "[CONFIG]");
   *   ListConfigPackagesResponse response = overTheAirServiceClient.listConfigPackages(formattedName);
   * }
   * </code></pre>
   *
   * @param name Relative resource name. For example: "deployments/deployment_x/configs/config_c"
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final ListConfigPackagesResponse listConfigPackages(String name) {
    CONFIG_PATH_TEMPLATE.validate(name, "listConfigPackages");
    ListConfigPackagesRequest request =
        ListConfigPackagesRequest.newBuilder().setName(name).build();
    return listConfigPackages(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Lists packages of the config. Returns `NOT_FOUND` if the matching path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatConfigName("[DEPLOYMENT]", "[CONFIG]");
   *   ListConfigPackagesRequest request = ListConfigPackagesRequest.newBuilder()
   *     .setName(formattedName)
   *     .build();
   *   ListConfigPackagesResponse response = overTheAirServiceClient.listConfigPackages(request);
   * }
   * </code></pre>
   *
   * @param request The request object containing all of the parameters for the API call.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final ListConfigPackagesResponse listConfigPackages(ListConfigPackagesRequest request) {
    return listConfigPackagesCallable().call(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Lists packages of the config. Returns `NOT_FOUND` if the matching path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatConfigName("[DEPLOYMENT]", "[CONFIG]");
   *   ListConfigPackagesRequest request = ListConfigPackagesRequest.newBuilder()
   *     .setName(formattedName)
   *     .build();
   *   ApiFuture&lt;ListConfigPackagesResponse&gt; future = overTheAirServiceClient.listConfigPackagesCallable().futureCall(request);
   *   // Do something
   *   ListConfigPackagesResponse response = future.get();
   * }
   * </code></pre>
   */
  public final UnaryCallable<ListConfigPackagesRequest, ListConfigPackagesResponse>
      listConfigPackagesCallable() {
    return stub.listConfigPackagesCallable();
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Update the packages under the config. Returns `NOT_FOUND` if the matching path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatConfigName("[DEPLOYMENT]", "[CONFIG]");
   *   List&lt;PackageConfig&gt; packageConfigs = new ArrayList&lt;&gt;();
   *   UpdateConfigPackagesResponse response = overTheAirServiceClient.updateConfigPackages(formattedName, packageConfigs);
   * }
   * </code></pre>
   *
   * @param name Relative resource name. For example: "deployments/deployment_x/configs/config_c"
   * @param packageConfigs Required. The names of the packages under the config.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final UpdateConfigPackagesResponse updateConfigPackages(
      String name, List<PackageConfig> packageConfigs) {
    CONFIG_PATH_TEMPLATE.validate(name, "updateConfigPackages");
    UpdateConfigPackagesRequest request =
        UpdateConfigPackagesRequest.newBuilder()
            .setName(name)
            .addAllPackageConfigs(packageConfigs)
            .build();
    return updateConfigPackages(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Update the packages under the config. Returns `NOT_FOUND` if the matching path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatConfigName("[DEPLOYMENT]", "[CONFIG]");
   *   List&lt;PackageConfig&gt; packageConfigs = new ArrayList&lt;&gt;();
   *   UpdateConfigPackagesRequest request = UpdateConfigPackagesRequest.newBuilder()
   *     .setName(formattedName)
   *     .addAllPackageConfigs(packageConfigs)
   *     .build();
   *   UpdateConfigPackagesResponse response = overTheAirServiceClient.updateConfigPackages(request);
   * }
   * </code></pre>
   *
   * @param request The request object containing all of the parameters for the API call.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final UpdateConfigPackagesResponse updateConfigPackages(
      UpdateConfigPackagesRequest request) {
    return updateConfigPackagesCallable().call(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Update the packages under the config. Returns `NOT_FOUND` if the matching path does not exist.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatConfigName("[DEPLOYMENT]", "[CONFIG]");
   *   List&lt;PackageConfig&gt; packageConfigs = new ArrayList&lt;&gt;();
   *   UpdateConfigPackagesRequest request = UpdateConfigPackagesRequest.newBuilder()
   *     .setName(formattedName)
   *     .addAllPackageConfigs(packageConfigs)
   *     .build();
   *   ApiFuture&lt;UpdateConfigPackagesResponse&gt; future = overTheAirServiceClient.updateConfigPackagesCallable().futureCall(request);
   *   // Do something
   *   UpdateConfigPackagesResponse response = future.get();
   * }
   * </code></pre>
   */
  public final UnaryCallable<UpdateConfigPackagesRequest, UpdateConfigPackagesResponse>
      updateConfigPackagesCallable() {
    return stub.updateConfigPackagesCallable();
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Lists all the configs that use the specified package.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatPackagePathName("[DEPLOYMENT]", "[PACKAGE_PATH]");
   *   ListConfigsUsingPackageResponse response = overTheAirServiceClient.listConfigsUsingPackage(formattedName);
   * }
   * </code></pre>
   *
   * @param name Relative resource name. For example: "deployments/deployment_x/packages/package_a"
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final ListConfigsUsingPackageResponse listConfigsUsingPackage(String name) {
    PACKAGE_PATH_PATH_TEMPLATE.validate(name, "listConfigsUsingPackage");
    ListConfigsUsingPackageRequest request =
        ListConfigsUsingPackageRequest.newBuilder().setName(name).build();
    return listConfigsUsingPackage(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Lists all the configs that use the specified package.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatPackagePathName("[DEPLOYMENT]", "[PACKAGE_PATH]");
   *   ListConfigsUsingPackageRequest request = ListConfigsUsingPackageRequest.newBuilder()
   *     .setName(formattedName)
   *     .build();
   *   ListConfigsUsingPackageResponse response = overTheAirServiceClient.listConfigsUsingPackage(request);
   * }
   * </code></pre>
   *
   * @param request The request object containing all of the parameters for the API call.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final ListConfigsUsingPackageResponse listConfigsUsingPackage(
      ListConfigsUsingPackageRequest request) {
    return listConfigsUsingPackageCallable().call(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD
  /**
   * Lists all the configs that use the specified package.
   *
   * <p>Sample code:
   *
   * <pre><code>
   * try (OverTheAirServiceClient overTheAirServiceClient = OverTheAirServiceClient.create()) {
   *   String formattedName = OverTheAirServiceClient.formatPackagePathName("[DEPLOYMENT]", "[PACKAGE_PATH]");
   *   ListConfigsUsingPackageRequest request = ListConfigsUsingPackageRequest.newBuilder()
   *     .setName(formattedName)
   *     .build();
   *   ApiFuture&lt;ListConfigsUsingPackageResponse&gt; future = overTheAirServiceClient.listConfigsUsingPackageCallable().futureCall(request);
   *   // Do something
   *   ListConfigsUsingPackageResponse response = future.get();
   * }
   * </code></pre>
   */
  public final UnaryCallable<ListConfigsUsingPackageRequest, ListConfigsUsingPackageResponse>
      listConfigsUsingPackageCallable() {
    return stub.listConfigsUsingPackageCallable();
  }

  @Override
  public final void close() {
    stub.close();
  }

  @Override
  public void shutdown() {
    stub.shutdown();
  }

  @Override
  public boolean isShutdown() {
    return stub.isShutdown();
  }

  @Override
  public boolean isTerminated() {
    return stub.isTerminated();
  }

  @Override
  public void shutdownNow() {
    stub.shutdownNow();
  }

  @Override
  public boolean awaitTermination(long duration, TimeUnit unit) throws InterruptedException {
    return stub.awaitTermination(duration, unit);
  }
}
